#!/usr/bin/env python3

from socket import AF_PACKET, SOCK_RAW, AF_INET, SOCK_DGRAM
from socket import socket, htons
from frame import Frame, MessageFrame

import fcntl
import struct

import click


def getHwAddr(ifname):
    s = socket(AF_INET, SOCK_DGRAM)
    info = fcntl.ioctl(s.fileno(), 0x8927,  struct.pack('256s', bytes(ifname, 'utf-8')[:15]))
    return ''.join('%02x' % b for b in info[18:24])


@click.group()
def cli():
    pass


@click.command()
@click.option('--interface', required=True, help="Ethernet interface")
@click.option('--destination', default="ff:ff:ff:ff:ff:ff", help="Destination du message")
@click.option('--source', default="ff:ff:ff:ff:ff:ff", help="Source du message")
@click.argument('object')
@click.argument('message')
def send_message(interface, source, destination, object, message):
    """ Envoie un message contenant le message MESSAGE avec l'objet OBJET par ethernet """
    s = socket(AF_PACKET, SOCK_RAW)
    # TODO: interface as argument or autodiscovered
    s.bind((interface, 0))
    pckt = MessageFrame(source, destination, object, message)
    s.send(pckt())

@click.command()
@click.option('--interface', required=True, help="Ethernet interface")
@click.option('--receive-all', help="Écoute pour toutes les adresses MAC")
def receive_message(interface, receive_all):
    """ Attend un message"""
    s = socket(AF_PACKET, SOCK_RAW, htons(0xbe00))
    # TODO: interface as argument or autodiscovered
    s.bind((interface, 0))
    while(True):

        message = s.recv(MessageFrame.len())
        message = MessageFrame.from_hex(message)
        if (receive_all
                or message.dest == getHwAddr(interface)
                or message.dest == '000000000000'):
            print(message)



@click.command()
@click.option('--interface', required=True, help="Ethernet interface")
@click.argument('frame')
def raw(interface, frame):
    """ Envoie la trame FRAME"""
    s = socket(AF_PACKET, SOCK_RAW)
    # TODO: interface as argument or autodiscovered
    s.bind((interface, 0))
    pckt = Frame(frame)
    s.send(pckt())


if __name__ == "__main__":

    cli.add_command(receive_message)
    cli.add_command(send_message)
    cli.add_command(raw)
    cli()
