import toml

config = toml.load("config.toml")
len_obj = config['message']['len_obj']
len_message = config['message']['len_message']

def complete_string(string, total_len):
    if total_len < len(string):
        raise ValueError("""La longueur de la chaine est supérieure à la
                            longueur totale""")
    string += ' ' * (total_len - len(string))
    return string


class Frame(bytearray):
    size = None
    text = None

    def __init__(self,
                 text="0123456789abcdef0123456789abcdef"):

        if len(text) % 2 != 0:
            raise ValueError("Le nombre de lettres doit etre pair")

        if isinstance(text, bytearray):
            self.hex_text = text
        else:
            self.hex_text = bytearray.fromhex(text)

        if len(self.hex_text) < 14:
            raise ValueError("Le nombre d'octets doit etre supérieur à 14")


    def __call__(self):
        return bytearray(self.hex_text)


class MessageFrame(Frame):
    def __init__(self,
                 source=None,
                 dest=None,
                 obj="plouf",
                 message="plif"):

        self.source = source
        self.dest = dest
        self.message = message
        self.obj = obj

        super().__init__(text=self.to_hex())


    def to_hex(self):
        if self.source is not None and self.dest is not None:
            source = bytearray.fromhex(self.source.replace(':', ' '))
            dest = bytearray.fromhex(self.dest.replace(':', ' '))
        else:
            raise ValueError("source and dest should be defined")

        if len(self.obj) > len_obj:
            raise ValueError(("L'objet doit contenir moins de {} caractères"
                              .format(len_obj)))
        obj = complete_string(self.obj, len_obj)

        if len(self.obj) > len_message:
            raise ValueError(("Le message doit contenir moins de {} caractères"
                              .format(len_message)))
        message = complete_string(self.message, len_message)

        return (dest
                + source
                + bytearray.fromhex('be 00')
                + bytearray(obj, encoding="utf-8")
                + bytearray(message, encoding="utf-8"))

    @classmethod
    def from_hex(cls, hexa):
        dest = hexa[:6].hex()
        source = hexa[6:12].hex()
        # protocol = hexa[12:13]
        obj = hexa[14:14 + len_obj].decode(encoding='utf-8')
        message = hexa[(14 + len_obj):(14 + len_message)].decode(encoding='utf-8')
        return cls(source, dest, obj, message)

    @staticmethod
    def len():
        return len_message + len_obj + 14

    def __str__(self):
        return ("Objet: {}\n"
                "Message : {}\n"
                "").format(self.obj, self.message)


class LedFrame(Frame):

    def __init__(self, name, state):
        # TODO
        # - max name length config, state length (1 byte ?)
        super().__init__('plouf')
